  import { world, MolangVariableMap } from "@minecraft/server";
  import { ActionFormData } from "@minecraft/server-ui";
  import { morphs } from 'morph_item.js';
  import { mode, mob_choosen_mode2, item_collect_mobs_indirectly } from 'setup_mode.js';
  import {addMorph} from 'interactMobs.js';
  
  let checkCreateData = false;
  let data = [];
  let menu;
  
  export class PaItem {
      constructor() {
          this.onUse = this.onUse.bind(this);
          this.onBeforeDurabilityDamage = this.onBeforeDurabilityDamage.bind(this);
      };
      onUse(event) {
          const { itemStack, source } = event;
          activateModeFromItem(itemStack, source)
      };
      onBeforeDurabilityDamage(event) {
          event.durabilityDamage = 0;
      };
  };
  
  
  function activateModeFromItem(itemStack, source) {
      addMenuItem(itemStack);
      
      if (mode == 0) {
          if (itemStack.getDynamicProperty("morphs") != undefined) {
              menu.show(source).then(item => {
                  if (!item.canceled) {
                      source.triggerEvent(data[item.selection].id);
                  }
              });
          }
          else {
              world.sendMessage(`Collect mobs by INTERACT WITH MOBS to unlock the menu.`);
          }
          collectMobIndirecly(source, itemStack);
      }
      else if (mode == 1) {
          const dimension = world.getDimension("overworld");
          const mobs = dimension.getEntities({ type: mob_choosen_mode2 });
  
          menu.show(source).then(item => {
              if (!item.canceled) {
                  for (const mob of mobs) {
                      mob.triggerEvent(morphs[item.selection].id);
                  }
              }
          });
  
  
      }
  }
  
  
  function addMenuItem(itemStack) {
      if (itemStack.getDynamicProperty("morphs") != undefined && mode == 0) {
          data = JSON.parse(itemStack.getDynamicProperty("morphs")).map(id => morphs.find(morph => morph.id == id)).filter(morph => morph != undefined);
          menu = new ActionFormData().title({ translate: "pa item" }).body("pa:morph_menu");
          for (const morph of data) {
              menu.button(morph.id, morph.icon ? morph.icon : `textures/icons/morph_menu/${morph.entity.split(":")[1]}/${morph.id.split(":")[1]}`);
          };
          checkCreateData = !checkCreateData;
      }
      else if (mode == 1 && !checkCreateData) {
          menu = new ActionFormData().title({ translate: "pa item" }).body("pa:morph_menu");
          for (const morph of morphs) {
              menu.button(morph.id, morph.icon ? morph.icon : `textures/icons/morph_menu/${morph.entity.split(":")[1]}/${morph.id.split(":")[1]}`);
          };
          checkCreateData = !checkCreateData;
  
      }
  }
  
  // world.afterEvents.itemUse.subscribe((event) => {
  //     let player = event.source;
  //     let item = event.itemStack;
  
  //     collectMobIndirecly(player, item);
  
  
  //     // activateMobToMobs(item);
  
  // });

  function collectMobIndirecly(player, item) {
      if (item.typeId === 'pa:pa item' && item_collect_mobs_indirectly) {
          let radius = 1000;  
          let soulCounter = {count:0};
  
          let nearbyEntities = player.dimension.getEntities({
              maxDistance: radius,
              location: player.location,
              excludeTypes: ["minecraft:player"]
          });
          
          for (let entity of nearbyEntities) {
              let morph = player.getMorph(entity);
              if (morph) {
  
                  if (addMorph(player, morph.id, soulCounter)) {
                      const direction = {
                          x: player.location.x - entity.getHeadLocation().x,
                          y: (player.location.y + 0.5) - entity.getHeadLocation().y,
                          z: player.location.z - entity.getHeadLocation().z
                      };
                      const variables = new MolangVariableMap();
                      variables.setVector3("variable.direction", direction);
                      //Math.sqrt căn bậc hai
                      //Math.pow là mũ 
                      variables.setFloat("variable.speed", Math.sqrt(Math.pow(direction.x, 2) + Math.pow(direction.y, 2) + Math.pow(direction.z, 2)) * 2);
  
                      //Tạo particle tại vị trí enetity
                      entity.dimension.spawnParticle("pa:soul_orb", entity.getHeadLocation(), variables);
                  }
              }
          }
          world.sendMessage(`You have collected ${soulCounter.count} mobs`);
          soulCounter.count = 0;
      }
  }
  