import { world, system } from "@minecraft/server";

function getCardinalRotation(yaw) {
  yaw = ((yaw % 360) + 360) % 360;
  return Math.round(yaw / 90) * 90 % 360;
}

world.afterEvents.itemUse.subscribe((event) => {
  const player = event.source;
  if (event.itemStack.typeId !== "dg:d2") return;

  // snap to block‐center
  const loc = player.location;
  const newPos = {
    x: Math.floor(loc.x) + 0.5,
    y: Math.floor(loc.y) + 0.0001,
    z: Math.floor(loc.z) + 0.5
  };

  const rotation = player.getRotation();
  const newYaw = getCardinalRotation(rotation.y);

  // first teleport immediately, with no-clipping
  system.runTimeout(() => {
    player.teleport(
      newPos,
      {
        rotation: { x: 0, y: newYaw },
        checkForBlocks: false,     // <- important
        keepVelocity: false
      }
    );

    // second teleport one or two ticks later, also no-clip
    system.runTimeout(() => {
      player.teleport(
        newPos,
        {
          rotation: { x: 0, y: newYaw },
          checkForBlocks: false,   // <- add this!
          keepVelocity: false
        }
      );
    }, 2);
  }, 0);
});
